/*
 * $Id: kl_buffered_output.h,v 1.5 2001/01/17 02:50:28 laik Exp $
 *
 * Copyright (C) 2000 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __BUFFERED_OUTPUT_H__
#define __BUFFERED_OUTPUT_H__

#include "kl_exception.h"
#include "kl_event.h"

struct KLBufferedOutput;
typedef struct KLBufferedOutput KLBufferedOutput;

/*
 * The destination of the output.
 */
typedef enum KLBufferedOutputType {
    KLBufferedOutputTypeFile,
    KLBufferedOutputTypeTerminal,
    KLBufferedOutputTypeSocket,
} KLBufferedOutputType;

/*
 * The conditions under which the buffer should be flushed.
 */
typedef enum KLBufferedOutputFlushFlag {
    KLBufferedOutputFlushFlagTimeout = 0x1,
    KLBufferedOutputFlushFlagSize = 0x2,
    KLBufferedOutputFlushFlagForce = 0x4,
} KLBufferedOutputFlushFlag;

/*
 * Infomation about the client.
 */
typedef enum KLBufferedOutputClientFlag {
    KLBufferedOutputClientFlagWritable = 0x1,
    KLBufferedOutputClientFlagReadable = 0x2,
    KLBufferedOutputClientFlagSharedBuffer = 0x4,
} KLBufferedOutputClientFlag;

struct KLBufferedOutputClient;
typedef struct KLBufferedOutputClient KLBufferedOutputClient;

/*
 * Callback for flushing the buffer.
 */
typedef KLException * (*KLBufferedOutputFunc)(
    KLBufferedOutput *bo, void *client_data);

/*
 * Callback for creating a new client.
 */
typedef KLException * (*KLBufferedOutputNewClientFunc)(
    KLBufferedOutputClient *client, void *client_data, void **new_client_data);

/*
 * Callback for deleting a client.
 */
typedef KLException * (*KLBufferedOutputDeleteClientFunc)(void *client_data);

/*
 * Callback for handling input.
 */
typedef KLException * (*KLBufferedOutputClientInputFunc)(
    KLBufferedOutput *bo, void *buf, unsigned int buf_len, void *client_data);

/*
 * Information specific to file buffers.
 */
typedef struct KLBufferedOutputDefTypeDataFile {
    const char *file_name;            /* File name to write to. */
    int fd;                           /* If no file name specified. */
} KLBufferedOutputDefTypeDataFile;

/*
 * Information specific to terminal buffers.
 */
typedef struct KLBufferedOutputDefTypeDataTerminal {
} KLBufferedOutputDefTypeDataTerminal;

/*
 * Information specific to socket buffers.
 */
typedef struct KLBufferedOutputDefTypeDataSocket {
    const char *listen_service;
} KLBufferedOutputDefTypeDataSocket;

typedef struct KLBufferedOutputDef {
    KLEventManager *manager;          /* Event Manager */
    KLBufferedOutputType type;        /* Type of output */
    union {
	KLBufferedOutputDefTypeDataFile file;
	KLBufferedOutputDefTypeDataTerminal terminal;
	KLBufferedOutputDefTypeDataSocket socket;
    } type_data;                      /* Type specific data */
    KLBufferedOutputFlushFlag flush_flags;
    struct timespec timeout;          /* Frequency of flushing */
    unsigned long flush_thresh;       /* Write flush threshhold */
    KLBufferedOutputFunc update;      /* Callback for update */
                                      /* Callback for new client */
    KLBufferedOutputNewClientFunc new_client; 
                                      /* Delete client */
    KLBufferedOutputDeleteClientFunc delete_client;
                                      /* Get input */
    KLBufferedOutputClientInputFunc input;
    void *client_data;                /* Data for callbacks */
    int verbosity;                    /* The verbosity of status messages. */
} KLBufferedOutputDef;

KLException *
KLBufferedOutputOpen(KLBufferedOutputDef *def, KLBufferedOutput **bo_p);

KLException *
KLBufferedOutputClose(KLBufferedOutput *bo);

KLException *
KLBufferedOutputPrintf(
    KLBufferedOutput *bo, KLBufferedOutputClient *client, const char *format, 
    ...) __attribute__ ((format (printf, 3, 4)));

KLException *
KLBufferedOutputWrite(
    KLBufferedOutput *bo, KLBufferedOutputClient *client, const void *buf,
    unsigned int buf_len);

KLException *
KLBufferedOutputFlush(KLBufferedOutput *bo);

void
KLBufferedOutputClientSetFlags(
    KLBufferedOutputClient *client, KLBufferedOutputClientFlag mask, 
    KLBufferedOutputClientFlag flags);

void
KLBufferedOutputTest(void);

#endif
