/*
 * $Id: kl_exception.h,v 1.5 2001/07/19 21:41:05 laik Exp $
 *
 * Copyright (C) 1993 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __EX_H__
#define __EX_H__

#include <errno.h>

/*
 * Some options to use when handling an exception.
 */
typedef enum KLExceptionFlag {
    KLExceptionFlagPrint = 0x1,
    KLExceptionFlagFatal = 0x2,
} KLExceptionFlag;

/*
 * The type of an exception.
 */
typedef struct KLExceptionClass {
    const char *format;
} KLExceptionClass;

/*
 * An instance of an exception.
 */
struct KLException;
typedef struct KLException KLException;

/*
 * A type of function to handle an exception.
 */
typedef void
(*KLExceptionHandlerFunc)(const KLException *exception, void *client_data);

/*
 * A mapping from exception types to functions that handle the exception.
 */
typedef struct KLExceptionMapping {
    KLExceptionClass *ex_class;
    KLExceptionHandlerFunc handler;
} KLExceptionMapping;

/* The system exception classes indexed by errno value. */
extern KLExceptionClass KLExceptionSysClass[];

/*
 * Throw an exception of the specified class with optional 
 * exception instance-specific data.
 */
#define KLExceptionThrow(_ex_class, _args...) \
    __ExceptionThrow(_ex_class, NULL, \
                     __FILE__, __FUNCTION__, __LINE__ , ## _args)

/*
 * Throw a system call exception with optional exception instance-specific
 * data.
 */
#define KLExceptionSysThrow(_ex_sys_call, _format, _args...) \
    ({ \
        int _rval0 = _ex_sys_call; \
        _rval0 >= 0 ? NULL : __ExceptionSysThrow( \
            errno, NULL, __FILE__, #_ex_sys_call, __LINE__, \
            _format , ## _args); \
    })    

/*
 * Same as above, but for system calls that return pointers.
 */
#define KLExceptionPtrSysThrow(_ex_sys_call, _format, _args...) \
    ({ \
        void *_rval1 = _ex_sys_call; \
        _rval1 != NULL ? NULL : __ExceptionSysThrow( \
            errno, NULL, __FILE__, #_ex_sys_call, __LINE__, \
            _format , ## _args); \
    })    

/*
 * Pass an exception up the call stack if an exception occurs.
 */
#define KLExceptionPass(_ex_class, _exception, _args...)  \
    ({ \
        KLException *_ex_result0 = _exception; \
        _ex_result0 == NULL ? NULL : __ExceptionThrow( \
            _ex_class, _ex_result0, \
            __FILE__, __FUNCTION__, __LINE__ , ## _args); \
    })

/*
 * Handle an exception in the default way: print out its meaning and
 * exit the program.
 */
#define KLExceptionErrorCatch(_exception, _fmt, args...)  \
    do { \
        KLException *_ex_result1 = _exception; \
        if (_ex_result1 != NULL) { \
            __ExceptionHandle(KLExceptionFlagPrint | KLExceptionFlagFatal, \
			      _ex_result1, __FILE__, __FUNCTION__, __LINE__, \
                              _fmt , ## args); \
	} \
    } while (0);

/*
 * Handle an exception by mapping the exception class to a handler function
 * and then calling that function.
 */
#define KLExceptionHandle(_ex_flags, _exception, _fmt, _args...)  \
    do { \
        KLException *_ex_result2 = _exception; \
        if (_ex_result2 != NULL) { \
            __ExceptionHandle(_ex_flags, _ex_result2, __FILE__, __FUNCTION__, \
                              __LINE__, _fmt , ## _args); \
	} \
    } while (0);

/*
 * Handle an exception by immediately aborting. This shouldn't be used
 * very much because it prevents higher layers from properly freeing
 * up resources.
 */
#define KLExceptionFatal(_ec_class, _fmt, _args...)  \
    do { \
        __ExceptionHandle(KLExceptionFlagPrint | KLExceptionFlagFatal, \
            KLExceptionThrow(_ec_class), \
            __FILE__, __FUNCTION__, __LINE__, _fmt, ## _args); \
    } while (0);

/*
 * Handle a system exception by immediately aborting. This shouldn't be
 * used very much because it prevents higher layers from properly freeing
 * up resources.
 */
#define KLExceptionSysFatal(_ex_sys_call, _fmt, _args...)  \
    do { \
        int _rval2 = _ex_sys_call; \
        if (_rval2 < 0) { \
            __ExceptionHandle(KLExceptionFlagPrint | KLExceptionFlagFatal, \
                KLExceptionSysThrow(_rval2, " "), \
                __FILE__, __FUNCTION__, __LINE__, _fmt, ## _args); \
        } \
    } while (0);

const KLExceptionClass *
KLExceptionGetClass(const KLException *exception);

const char *
KLExceptionGetMeaning(const KLException *exception);

void
KLExceptionFree(KLException *exception);

/*
 * These functions should not be called directly.
 */
KLException *
__ExceptionThrow(
    KLExceptionClass *ex_class, KLException *next, const char *file_name,
    const char *func_name, unsigned int line_num, ...);

KLException *
__ExceptionSysThrow(
    int error_number, KLException *next,
    const char *file_name, const char *func_name, unsigned int line_num,
    const char *format, ...) /* __attribute__ ((format (printf, 6, 7)))*/; 

void
__ExceptionHandle(
    KLExceptionFlag flags, KLException *exception,
    const char *file_name, const char *func_name, unsigned int line_num,
    const char *fmt, ...) /*__attribute__ ((format (printf, 6, 7)))*/;

#ifdef TEST

void
KLExceptionTest(void);

#endif

#endif

