/*
 * $Id: kl_network_protocol.h,v 1.4 2001/02/13 07:50:33 laik Exp $
 *
 * This file contains interfaces and data structures for the flow module.
 *
 * Copyright (C) 1998 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NETWORK_PROTOCOL_H__
#define __NETWORK_PROTOCOL_H__

#include "kl_util.h"
#include "kl_array.h"

struct KLNetworkProtocol;
typedef struct KLNetworkProtocol KLNetworkProtocol;

struct KLNetworkProtocolSet;
typedef struct KLNetworkProtocolSet KLNetworkProtocolSet;

typedef struct KLNetworkProtocolPacketKey {
    u_int32_t key[7];
    unsigned long flow_valid;
    unsigned long packet_valid;
} KLNetworkProtocolPacketKey;

typedef struct KLNetworkProtocolPacket {
    const KLNetworkProtocol *protocols[7];
    const u_int8_t *buffer;
    unsigned int buf_len;
    unsigned int header_offsets[7];
} KLNetworkProtocolPacket;

typedef enum KLNetworkProtocolMatch {
    KLNetworkProtocolMatchNone,
    KLNetworkProtocolMatchExact,
    KLNetworkProtocolMatchCumulative,
} KLNetworkProtocolMatch; 

/*
 * If this packet is owned by this protocol, get the packet headers.
 * Otherwise return false.
 */
typedef bool (KLNetworkProtocolGetPacketHeaderFun)(
    KLNetworkProtocolPacket *packet, unsigned int index, void *client_data);

/*
 * Makes a new packet key.
 */
typedef bool (KLNetworkProtocolGetPacketKeyFun)(
    const KLNetworkProtocolPacket *packet, unsigned int index,
    KLNetworkProtocolPacketKey *key, void *client_data);

/*
 * Makes a new ack key.
 */
typedef bool (KLNetworkProtocolGetAckKeyFun)(
    const KLNetworkProtocolPacket *packet, unsigned int index,
    KLNetworkProtocolPacketKey *key, void *client_data);

/*
 * Makes a new acked key.
 */
typedef bool (KLNetworkProtocolGetAckedKeyFun)(
    const KLNetworkProtocolPacket *packet, unsigned int index,
    KLNetworkProtocolPacketKey *key, void *client_data);

/*
 * Makes a new retransmission key.
 */
typedef bool (KLNetworkProtocolGetRetransmissionKeyFun)(
    const KLNetworkProtocolPacket *packet, unsigned int index,
    KLNetworkProtocolPacketKey *key, void *client_data);

/*
 * Determines if packet and ack match.
 */
typedef KLNetworkProtocolMatch (KLNetworkProtocolMatchAnyAckFun)(
    const KLNetworkProtocolPacket *packet, unsigned int packet_index,
    const KLNetworkProtocolPacket *ack, unsigned int ack_index,
    void *client_data);

/*
 * Determines if packet and ack match.
 */
typedef KLNetworkProtocolMatch (KLNetworkProtocolAnyMatchAckFun)(
    const KLNetworkProtocolPacket *packet, unsigned int packet_index,
    const KLNetworkProtocolPacket *ack, unsigned int ack_index,
    void *client_data);

/*
 * Determines if two packets are retransmissions.
 */
typedef bool (KLNetworkProtocolMatchRetransmissionFun)(
    const KLNetworkProtocolPacket *original_packet,
    unsigned int original_index, const KLNetworkProtocolPacket *new_packet, 
    unsigned int new_index, void *client_data);

/*
 * Compose a new packet.
 */
typedef bool (KLNetworkProtocolComposePacketFun)(
    u_int8_t *packet_buffer, unsigned int buffer_len,
    void *packet_client_data, void *client_data);

/*
 * Stringify a packet.
 */
typedef bool (KLNetworkProtocolPacketGetStringFun)(
    const KLNetworkProtocolPacket *packet, unsigned int packet_index,
    KLArray *array, void *client_data);

/*
 * Get packet size.
 */
typedef bool (KLNetworkProtocolPacketGetSizeFun)(
    const KLNetworkProtocolPacket *packet, unsigned int packet_index,
    unsigned int *size, void *client_data);

/*
 * Free protocol.
 */
typedef void (KLNetworkProtocolFreeFun)(void *client_data);

typedef struct KLNetworkProtocolDef {
    KLNetworkProtocolGetPacketHeaderFun *get_packet_header;
    KLNetworkProtocolGetPacketKeyFun *get_packet_key;
    KLNetworkProtocolGetAckKeyFun *get_ack_key;
    KLNetworkProtocolGetAckedKeyFun *get_acked_key;
    KLNetworkProtocolGetRetransmissionKeyFun *get_retransmission_key;
    KLNetworkProtocolMatchAnyAckFun *match_any_ack;
    KLNetworkProtocolAnyMatchAckFun *any_match_ack;
    KLNetworkProtocolMatchRetransmissionFun *match_retransmission;
    KLNetworkProtocolComposePacketFun *compose_packet;
    KLNetworkProtocolPacketGetStringFun *get_string;
    KLNetworkProtocolPacketGetSizeFun *packet_get_size;
    KLNetworkProtocolFreeFun *free_fun;
    void *client_data;
} KLNetworkProtocolDef;

KLNetworkProtocolSet *
KLNetworkProtocolSetNew(void);

void
KLNetworkProtocolSetFree(KLNetworkProtocolSet *set);

void
KLNetworkProtocolSetAddProtocol(
    KLNetworkProtocolSet *set, KLNetworkProtocol *protocol);

KLNetworkProtocol *
KLNetworkProtocolNew(KLNetworkProtocolDef *protocol_def);

void
KLNetworkProtocolFree(KLNetworkProtocol *protocol);

void
KLNetworkProtocolSetClientData(
    KLNetworkProtocol *protocol, void *client_data);

bool
KLNetworkProtocolSetGetProtocol(
    KLNetworkProtocolSet *set, const u_int8_t *buffer, unsigned int buf_len, 
    KLNetworkProtocolPacket *packet);

void
KLNetworkProtocolSetClientData(
    KLNetworkProtocol *protocol, void *client_data);

bool
KLNetworkProtocolGetPacketHeader(
    KLNetworkProtocolPacket *packet, unsigned int index);

bool
KLNetworkProtocolGetPacketKey(
    const KLNetworkProtocolPacket *packet, unsigned int index,
    KLNetworkProtocolPacketKey *key);

bool
KLNetworkProtocolGetAckKey(
    const KLNetworkProtocolPacket *packet, unsigned int index,
    KLNetworkProtocolPacketKey *key);

bool
KLNetworkProtocolGetAckedKey(
    const KLNetworkProtocolPacket *packet, unsigned int index,
    KLNetworkProtocolPacketKey *key);

bool
KLNetworkProtocolGetRetransmissionKey(
    const KLNetworkProtocolPacket *packet, unsigned int index,
    KLNetworkProtocolPacketKey *key);

KLNetworkProtocolMatch
KLNetworkProtocolMatchAnyAck(
    const KLNetworkProtocolPacket *packet, unsigned int packet_index,
    const KLNetworkProtocolPacket *ack, unsigned int ack_index);

KLNetworkProtocolMatch
KLNetworkProtocolAnyMatchAck(
    const KLNetworkProtocolPacket *packet, unsigned int packet_index,
    const KLNetworkProtocolPacket *ack, unsigned int ack_index);

bool
KLNetworkProtocolMatchRetransmission(
    const KLNetworkProtocolPacket *original_packet, unsigned int original_index,
    const KLNetworkProtocolPacket *new_packet, unsigned int new_index);

bool
KLNetworkProtocolComposePacket(
    KLNetworkProtocol *protocol, u_int8_t *packet_buffer,
    unsigned int buf_len, void *client_data);

bool
KLNetworkProtocolPacketGetString(
    const KLNetworkProtocolPacket *packet, 
    unsigned int packet_index, KLArray *array);

bool
KLNetworkProtocolPacketGetSize(
    const KLNetworkProtocolPacket *packet,
    unsigned int packet_index, unsigned int *size);

#endif
