/*
 * $Id: kl_parse_option.h,v 1.5 2001/06/20 21:18:28 laik Exp $
 *
 * Command-line option parsing.
 *
 * Copyright (C) 1993 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __PARSE_OPTION_H__
#define __PARSE_OPTION_H__

#include <getopt.h>

#include "kl_exception.h"

struct KLParseOptionServers;
typedef struct KLParseOptionServers KLParseOptionServers;

typedef enum KLParseOptionType {
    PO_TYPE_INT, PO_TYPE_STRING, PO_TYPE_DOUBLE, PO_TYPE_SCALAR,
} KLParseOptionType;

#define PO_TYPE_LAST PO_TYPE_SCALAR + 1

/*
 * This type represents the value of a command line argument.
 */
typedef union KLParseOptionValue {
    long i;
    char *s;
    double d;
    void *a;
} KLParseOptionValue;

/*
 * This is a type where a string denotes a scalar value.
 * e.g. "Monday" = 0, "Tuesday" = 1, etc.
 */
typedef struct KLParseOptionScalarPair {
    const char *name;
    void *client_data;
} KLParseOptionScalarPair;

typedef enum KLParseOptionTakesArg {
    PO_NO_ARG = no_argument,
    PO_REQUIRED_ARG = required_argument,
    PO_OPTIONAL_ARG = optional_argument
} KLParseOptionTakesArg;

typedef enum KLParseOptionRequired {
    PO_REQUIRED, PO_NOT_REQUIRED
} KLParseOptionRequired;

/*
 * This type is used to specify information about a command line
 * option.
 */
typedef struct KLParseOptionSpec {
    const char *name;                    /* The English name */
    const char option_char;              /* The char denoting this option */
    int id;                              /* A unique id for this option */
    enum KLParseOptionType type;         /* The type of this opt (see above) */
    const char *doc;                     /* Documentation */
    union KLParseOptionValue def_val;    /* The default value */
    union KLParseOptionValue no_arg_val; /* The value if there's no arg. */
    struct KLParseOptionScalarPair *scalar_pairs;
                                         /* The mapping of strs to scalars */
    enum KLParseOptionTakesArg takes_arg;/* Whether this option takes an arg */
    enum KLParseOptionRequired required; /* Whether this option is required */
} KLParseOptionSpec;

extern KLExceptionClass KLParseOptionECDuplicateOptionChar;
extern KLExceptionClass KLParseOptionECHelp;
extern KLExceptionClass KLParseOptionECOptionUnknown;
extern KLExceptionClass KLParseOptionECLongOptionUnknown;
extern KLExceptionClass KLParseOptionECArgumentRequired;
extern KLExceptionClass KLParseOptionECLongArgumentRequired;
extern KLExceptionClass KLParseOptionECArgumentInt;
extern KLExceptionClass KLParseOptionECArgumentDouble;
extern KLExceptionClass KLParseOptionECArgumentScalar;
extern KLExceptionClass KLParseOptionECOptionRequired;

KLException *
KLParseOptionParse(
    int argc, char **argv, const KLParseOptionSpec *client_specs,
    KLParseOptionValue **values_p, int *remaining);

KLException *
KLParseOptionHelp(const KLParseOptionSpec *specs);

void
KLParseOptionFree(const KLParseOptionSpec *specs, KLParseOptionValue *values);

KLParseOptionServers *
KLParseOptionServersParse(
    const char *server_names, const char *default_service);

void
KLParseOptionServersFree(KLParseOptionServers *servers);

int
KLParseOptionNumServers(const KLParseOptionServers *server);

void
KLParseOptionGetServer(
    const KLParseOptionServers *server, int server_index, 
    char **host, char **service);

#ifdef TEST
void
KLParseOptionTest(int main_argc, char **main_argv);
#endif

#endif
