/*
 * $Id: kl_table.h,v 1.6 2001/03/30 22:53:07 laik Exp $
 *
 * This file contains interfaces for routines which manage tables.
 *
 * Copyright (C) 1993 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TABLE_H__
#define __TABLE_H__

#include "kl_array.h"
#include "kl_exception.h"

struct KLTable;
typedef struct KLTable KLTable;

struct KLTableTuple;
typedef struct KLTableTuple KLTableTuple;

struct KLTableQuery;
typedef struct KLTableQuery KLTableQuery;

typedef enum KLTableSearchType {
    KLTableSearchEquals,
    KLTableSearchAtMost,
    KLTableSearchAtLeast
} KLTableSearchType;

/*
 * This function is used by the source table to tell the dependent
 * table that it has a new relevant tuple.
 */
typedef void (KLTableDepInsertFun)(
    struct KLTableTuple *tuple, void *client_data);

typedef void (KLTableDepDeleteFun)(
    struct KLTableTuple *tuple, void *client_data);

/*
 * This function is called when iterating over tuples.
 */
typedef KLException *(KLTableIterateFun)(
    struct KLTableTuple *tuple, void *client_data);

/*
 * This function is used to free tuple client data.
 */
typedef void (KLTableFreeClientDataFun)(
    void *client_data, void *tuple_client_data);

/*
 * These are flags for defining the table type.
 */
typedef enum KLTableFlag {
    KLTableFlagNone = 0,
    KLTableFlagDependent = 0x1,
    KLTableFlagCopying = 0x2,
} KLTableFlag;

typedef struct KLTableDef {
    KLTableFlag flags;                         /* Specify the type table. */
    unsigned int tuple_size;                 /* Dimensions of table. */
    KLTableQuery *query;                       /* Range of dependency. */
    KLTableDepInsertFun *insert;               /* Insert callback. */
    KLTableDepDeleteFun *delete;               /* Delete callback. */
    void *client_data;                       /* Clientdata for callbacks. */
    KLTableFreeClientDataFun *free_client_data; /* Free client data callback. */
} KLTableDef;

KLTable *
KLTableNew(KLTableDef *def);

void
KLTableNewFinish(KLTable *table);

void 
KLTableFree(struct KLTable *table);

unsigned int
KLTableGetDimensions(KLTable *table);

struct KLTableTuple *
KLTableNewTuple(struct KLTable *table, const double *v, void *client_data);

void 
KLTableDeleteTuple(KLTable *table, KLTableTuple *tuple);

KLTableTuple *
KLTableFirstTuple(const KLTable *table);

KLTableTuple *
KLTableNextTuple(KLTableTuple *tuple);

unsigned int
KLTableCount(const KLTable *table);

const double *
KLTableTupleGetValues(const KLTableTuple *tuple);

KLTableTuple *
KLTableTupleSetValues(
    KLTableTuple *tuple, unsigned long valid, const double *values);

void *
KLTableTupleGetData(const KLTableTuple *tuple);

void
KLTableTupleSetData(struct KLTableTuple *tuple, void *client_data);

void
KLTableTupleGetString(const struct KLTableTuple *tuple, KLArray *array);

KLException *
KLTableSearch(KLTable *table, const double *lower, const double *upper,
	    KLTableTuple **tuple_p);

KLTableTuple *
KLTableSelect(KLTable *table, unsigned int dimension, unsigned int rank);

void
KLTableGetString(KLTable *table, KLArray *array);

void
KLTableVerify(KLTable *table);

struct KLTableQuery *
KLTableQueryNew(KLTable *source_table);

void
KLTableQueryFree(KLTableQuery *query);

void
KLTableQueryInitialize(KLTableQuery *query);

void 
KLTableQueryAddClause(
    KLTableQuery *query, const double *lower, const double *upper);

KLException *
KLTableQueryIterate(KLTableQuery *query, KLTableIterateFun iterate_fun,
		  void *client_data);

void
KLTableQueryGetString(KLTableQuery *query, KLArray *array);

#ifdef TEST

void
KLTableTest(void);

#endif

#endif
