/*
 * $Id: kl_timing.h,v 1.4 2001/06/20 01:35:43 laik Exp $
 *
 * This file must be included in order to use the timing routines.
 *
 * Copyright (C) 1993 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TIMING_H__
#define __TIMING_H__

#include <math.h>
#include <sys/time.h>
#include <time.h>

#define BILLION 1000000000L

#ifndef clock_gettime
#define clock_gettime temp_clock_gettime
#define CLOCK_REALTIME 0
#endif

int
temp_clock_gettime(int which_clock, struct timespec *current_time);

/*
 * Performs the addition (Result = First + Second), performing the
 * appropriate carrying operations
 */
extern inline void
KLTimingAdd(struct timespec *result, const struct timespec *first,
	  const struct timespec *second)
	
{
    result->tv_nsec = first->tv_nsec + second->tv_nsec;
    result->tv_sec = first->tv_sec + second->tv_sec;
}

/*
 * Performs the subtraction (Result = First - Second), performing the
 * appropriate carrying operations
 */
extern inline void
KLTimingSubtract(struct timespec *result, const struct timespec *first,
	       const struct timespec *second)
	       
{
    result->tv_nsec = first->tv_nsec - second->tv_nsec;
    result->tv_sec = first->tv_sec - second->tv_sec;
}

/*
 * This function copies the contents of one timing to another.
 */
extern inline void
KLTimingCopy(struct timespec *dest, const struct timespec *src)
{
    dest->tv_sec = src->tv_sec;
    dest->tv_nsec = src->tv_nsec;
}

/*
 * This function sets components of the timing.
 */
extern inline void
KLTimingSet(struct timespec *ts, long seconds, long nano_seconds)
{
    ts->tv_sec = seconds;
    ts->tv_nsec = nano_seconds;
}

/*
 * This function gets the timing value as a nanosecond value.
 */
extern inline long
KLTimingGetNS(const struct timespec *ts)
{
    return ts->tv_nsec;
}

/*
 * This function gets the timing value as a nanosecond value.
 */
extern inline long
KLTimingGetSeconds(const struct timespec *ts)
{
    return ts->tv_sec;
}

/*
 * This function returns the time in seconds as a double.
 */
extern inline double
KLTimingGetDouble(const struct timespec *ts)
{
    return ts->tv_sec + ((double) ts->tv_nsec / BILLION);
}

/*
 * This function sets the timing value to that of a double precision
 * floating point value.
 */
extern inline void
KLTimingSetDouble(struct timespec *ts, double seconds)
{
    ts->tv_sec = seconds;
    ts->tv_nsec = rint((seconds - (double) ts->tv_sec) * 1000000000.0);
}

/*
 * This function sets the timing value to that of a millisecond value.
 */
extern inline void
KLTimingSetScaled(struct timespec *ts, long scale, long value)
{
    ts->tv_sec = value / scale;
    ts->tv_nsec = (value % scale) * (BILLION / scale);
}

void
KLTimingNormalize(struct timespec *ts);

int
KLTimingCompare(const struct timespec *First, const struct timespec *Second);

void
KLTimingTest(void);

#endif

