/*
 * $Id: kl_tree.h,v 1.4 2001/02/13 07:48:09 laik Exp $
 *
 * This file contains interfaces for an implementation of ternary trees.
 *
 * Copyright (C) 1993 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TREE_H__
#define __TREE_H__

#include "kl_array.h"
#include "kl_exception.h"

struct KLTree;
typedef struct KLTree KLTree;

struct KLTreeNode;
typedef struct KLTreeNode KLTreeNode;

typedef struct KLTreeStats {
    int node_count;
    int height;
} KLTreeStats;

typedef enum KLTreeKeyType {
    KLTreeKeyString,
    KLTreeKeyDouble,
    KLTreeKeyLong,
    KLTreeKeyCustom
} KLTreeKeyType;

typedef enum KLTreeSearchType {
    KLTreeSearchEquals,
    KLTreeSearchAtMost,
    KLTreeSearchAtLeast
} KLTreeSearchType;

/*
 * This specifies how we visit the tree. Nodes are only visited if
 * they meet all the requirements. These are flags, so more than
 * one criteria can be specified.
 */
typedef enum KLTreeVisitType {
    KLTreeVisitAny = 0x0,
    KLTreeVisitAtMost = 0x1,
    KLTreeVisitAtLeast = 0x2
} KLTreeVisitType;

/*
 * This function is called when iterating over items in a tree.
 * It needs to deal with possible null items. 
 */
typedef KLException * (*KLTreeVisitFun)(void *item, void *client_data);

/*
 * This function is used to free items.
 */
typedef void (*KLTreeFreeItemFun)(void *client_data, void *item);

void
KLTreeInit(KLTree *tree, KLTreeKeyType key_type, unsigned int num_keys,
	 KLTreeFreeItemFun free_item, void *client_data);

KLTree *
KLTreeNew(
    KLTreeKeyType key_type, unsigned int num_keys, KLTreeFreeItemFun free_item,
    void *client_data);

void
KLTreeFree(KLTree *tree);

unsigned long
KLTreeCount(KLTree *tree);

void *
KLTreeNodeGetItem(KLTreeNode *node);

void
KLTreeNodeSetItem(KLTreeNode *node, void *item);

KLTreeNode *
KLTreeInsertItem(KLTree *tree, const void *keys, void *item, int *new);

void
KLTreeFreeNode(KLTree *tree, KLTreeNode *node);

KLTreeNode *
KLTreeSearch(KLTree *tree, const void *keys, KLTreeSearchType search_type);

KLTreeNode *
KLTreeSelect(KLTree *tree, unsigned long rank);

KLException *
KLTreeVisit(
    KLTree *tree, const void *low_keys, const void *high_keys,
    KLTreeVisitType *visit_type, KLTreeVisitFun visit_fun, void *client_data);

void
KLTreeVerify(KLTree *tree);

void
KLTreeGetString(KLTree *tree, KLArray *array);

void
KLTreeGetStats(KLTree *tree, KLTreeStats *stats);

#ifdef TEST

void
KLTreeTest(void);

#endif

#endif
